import pandas as pd
import numpy as np
import json
import SharedArray as sa
from multiprocessing import Process, Pool, cpu_count
from scipy.stats import entropy
from math import log, factorial


def calculate_entropy_process(args):
    return calculate_entropy(*args)


def calculate_entropy(column, window_size, pattern_length, normalization):
    data = sa.attach('shm://' + column)

    length = len(data)
    entropy_list = np.zeros(length)
    distribution = pd.Series()

    left_margin_count = list(range(1, window_size+1))
    right_margin_count = list(reversed(range(1, window_size+1)))
    middle_count = [window_size] * (length - 2*window_size)
    count_array = np.array(left_margin_count + middle_count + right_margin_count)

    for window_start in range(0, length - window_size + 1):
        window_end = window_start + window_size
        if distribution.empty:
            values, counts = np.unique(data[window_start: window_end],
                                       return_counts=True)
            distribution = pd.Series(counts, index=values)

        else:
            last_value = data[window_start]
            next_value = data[window_end - 1]

            distribution[last_value] = distribution[last_value] - 1
            if not next_value in distribution.index:
                distribution[next_value] = 1
            else:
                distribution[next_value] += 1

        entropy_list[window_start: window_end] = entropy_list[window_start: window_end] + entropy(distribution.values, base=2)

    entropy_list = normalization(entropy_list/count_array, pattern_length)
    return column, entropy_list


def default_normalization(entropies, pattern_length):
    return entropies


def linear_normalization(entropies, pattern_length):
    if pattern_length <= 1:
        return default_normalization(entropies, pattern_length)
    else:
        return entropies/(pattern_length - 1)


def range_normalization(entropies, pattern_length):
    return entropies/log(factorial(pattern_length), 2)


def sliding_window_entropy(data: pd.DataFrame, window_size, pattern_length=1, normalization=default_normalization):

    pool = Pool(processes=cpu_count())

    arrays = [file.name.decode("utf-8") for file in sa.list()]
    for column, values in data.iteritems():
        if column in arrays:
            sa.delete('shm://' + column)
        a = sa.create('shm://' + column, len(values), dtype=values.dtype)
        a[:] = values

    args = [(column, window_size, pattern_length, normalization)
            for column in data.columns.values]
    entropy_dict = {}

    for column, entropy_list in pool.map(calculate_entropy_process, args):
        entropy_dict[column] = entropy_list
        sa.delete('shm://' + column)

    return pd.DataFrame(entropy_dict)
