import pandas as pd
import numpy as np
import json
import SharedArray as sa
from multiprocessing import Process, Pool, cpu_count


def decode(a: int, degree):
    result = []
    for i in range(degree):
        result.append(a % degree)
        a = a // degree
    return np.array(result).T


def calculate_permutations_process(args):
    return calculate_permutations(*args)


def calculate_permutations(column, timelag, gap):
    encoding_vector = np.power(np.full(timelag, timelag, dtype=np.int64), np.arange(timelag))
    data = sa.attach('shm://' + column)

    permutationList = []
    range_start = timelag + (timelag - 1) * gap
    range_end = len(data) + 1

    for subsequence_end in range(range_start, range_end):
        subsequence_start = subsequence_end - (timelag + (timelag - 1) * gap)
        permutation = data[subsequence_start: subsequence_end: gap + 1].argsort(kind='mergesort')
        permutationList.append(np.dot(permutation, encoding_vector))

    return column, permutationList


def calculate_ordinal_patterns(data: pd.DataFrame, t: 'timelag'=5, g: 'gap'=0):
    shortened_df = data[t + (t-1)*g - 1:].reset_index(drop=True)

    pool = Pool(processes=cpu_count())

    arrays = [file.name.decode("utf-8") for file in sa.list()]

    for column, values in data.iteritems():
        if column in arrays:
            sa.delete('shm://' + column)
        a = sa.create('shm://' + column, len(values), dtype=values.dtype)
        a[:] = values

    args = [(column, t, g) for column in data.columns.values]
    permutations_dict = {}

    for column, permutations in pool.map(calculate_permutations_process, args):
        permutations_dict[column] = permutations
        sa.delete('shm://' + column)

    permutations_df = pd.DataFrame(permutations_dict)
    return shortened_df, permutations_df
