import numpy as np
import pandas as pd
from scipy.stats import mode


def wrapped_mode(data, axis=0):
    mode_data = mode(data, axis=axis)[0]
    return mode_data.reshape(len(mode_data))


def scale_timeseries(data, scaling, scaling_function=np.mean):
    new_length = len(data)//scaling
    max_index = new_length*scaling
    reshaped_data = data[0:max_index].values.reshape((new_length, scaling))
    scaled_data = scaling_function(reshaped_data, axis=1)
    return scaled_data


def scale_dataframe(df, scaling, target):
    scaled_dict = {}

    for column in df.columns.values:
        if column != target:
            scaled_dict[column] = scale_timeseries(df[column], scaling)
        else:
            scaled_dict[column] = scale_timeseries(df[column], scaling,
                                                   wrapped_mode)

    return pd.DataFrame(scaled_dict)
