import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cm
from sklearn.preprocessing import LabelEncoder
from math import floor


def pattern_density_plot(data, step_width, title, encoder=None, boundaries=[]):

    if encoder is None:
        encoder = LabelEncoder().fit(data)

    transformed_data = encoder.transform(data)

    fig = plt.figure()
    axes = fig.add_subplot(111)
    axes.get_yaxis().set_visible(False)
    axes.set_title(title)

    img = np.zeros((len(encoder.classes_), floor(len(data) / step_width)))

    for counter in range(floor(len(data) / step_width)):
        x_start = step_width * counter
        x_end = step_width * (counter + 1)

        values, counts = np.unique(transformed_data[x_start:x_end],
                                   return_counts=True)
        probabilities = counts/counts.sum()
        img[values, counter] = probabilities

    img = 1 - img
    axes.imshow(img, extent=(0, len(data), 0, 0.5*len(data)),
                interpolation='none', cmap=cm.gray)

    if boundaries:
        plt.xticks(boundaries, ['x']*len(boundaries))

    fig.tight_layout()
    return fig
