import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.cm as cm
from sklearn.metrics import confusion_matrix


def _default_encode(x):
    return x


def accuracy_from_confusion(confusion, marginal):
    marginal_matrix = np.array([list(marginal)]*len(marginal))
    return (confusion.as_matrix() * marginal_matrix).diagonal().sum()


def multi_confusion_plot(confusions):
    fig = plt.figure(figsize=(50, 50 * len(confusions)))
    for i in range(0, len(confusions)):
        confusion = confusions[i]
        ax = fig.add_subplot(1, 10, i+1)
        ax.imshow(confusion.as_matrix(), interpolation='none', cmap=cm.gray)
        ax.get_yaxis().set_visible(False)
        ax.get_xaxis().set_visible(False)
    return fig


def confusion_plot(confusion_matrix, title):
    fig = plt.figure()
    axes = fig.add_subplot(111)
    axes.get_yaxis().set_visible(False)
    axes.set_title(title)

    img = confusion_matrix.as_matrix()
    axes.imshow(img, interpolation='none', cmap=cm.gray)

    fig.tight_layout()
    return fig



class Evaluation:

    def __init__(self, classifier, X_train, Y_train, X_test, Y_test, name,
                 encode=_default_encode):

        self.classifier = classifier

        self.X_train = encode(X_train)
        self.X_test = encode(X_test)
        self.Y_test = Y_test
        self.Y_train = Y_train
        self.name = name

    def evaluate(self):
        self.classifier.fit(self.X_train, self.Y_train)
        predicted = self.classifier.predict(self.X_test)
        cm = confusion_matrix(self.Y_test, predicted)
        cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]

        class_labels = np.unique(np.append(self.Y_test, predicted))
        cm_df = pd.DataFrame(columns=np.core.defchararray.add('predicted=', class_labels.astype(str)),
                             index=class_labels, data=cm)
        return cm_df
